unit GenericContext;

interface

	function Main (params: Ptr; param_size: longint; var result: Ptr; var result_size: longint; refcon: longint): OSErr;

implementation

	uses
		Processes, DialogSubs;


{# selectors for 'GNRC'  Generic Context checker}
{# 0,ostype -> test front application's creator}
{# 1,str -> test front window title}
{# 4,item,value -> test dialog control for highlight, true is highlight = value}
{# 5,item,value -> test dialog control for value, true if control value = value}
{# 6,item -> test dialog string, true if string not empty}

	const
		opFrontApplicationCreator = 0;
		opFrontWindowName = 1;
		opDialogItemHighlight = 4;
		opDialogItemValue = 5;
		opDialogItemTextEmpty = 6;

	function Main (params: Ptr; param_size: longint; var result: Ptr; var result_size: longint; refcon: longint): OSErr;

		function GetStr: Str255;
			var
				s: Str255;
				len: integer;
		begin
			s := StringPtr(params)^;
			GetStr := s;
			len := length(s);
			if odd(len) then begin
				len := len + 1;
			end; (* if *)
			params := Ptr(longint(params) + len);
		end; (* GetStr *)

		function GetOSType: OSType;
		begin
			GetOSType := OSTypePtr(params)^;
			params := Ptr(longint(params) + sizeof(OSType));
		end; (* GetOSType *)

		function GetInt: integer;
		begin
			GetInt := integerPtr(params)^;
			params := Ptr(longint(params) + sizeof(integer));
		end; (* GetInt *)

		function DoFrontApplicationCreator: boolean;
			var
				res: boolean;
				wanted_creator: OSType;
				psn: ProcessSerialNumber;
				pinfo: ProcessInfoRec;
				response: longint;
		begin
			res := false;
			wanted_creator := GetOSType;
			if (Gestalt(gestaltOSAttr, response) = noErr) & btst(response, gestaltLaunchControl) then begin
				if GetFrontProcess(psn) = noErr then begin
					pinfo.processInfoLength := sizeof(pinfo);
					pinfo.processName := nil;
					pinfo.processAppSpec := nil;
					if GetProcessInformation(psn, pinfo) = noErr then begin
						res := (pinfo.processSignature = wanted_creator);
					end; (* if *)
				end; (* if *)
			end; (* if *)
			DoFrontApplicationCreator := res;
		end; (* DoFrontApplicationCreator *)

		function DoFrontWindowName: boolean;
			var
				res: boolean;
				w: WindowPtr;
				tmpstr: Str255;
		begin
			res := false;
			w := FrontWindow;
			if w <> nil then begin
				tmpstr := WindowPeek(w)^.titleHandle^^;
				res := EqualString(GetStr, tmpstr, false, true);
			end; (* if *)
			DoFrontWindowName := res;
		end; (* DoFrontWindowName *)

		function DoDialogItem (var dlog: DialogPtr; var item: integer; var k: integer): boolean;
		begin
			DoDialogItem := false;
			dlog := FrontWindow;
			if (dlog <> nil) & (WindowPeek(dlog)^.windowKind = dialogKind) then begin
				item := GetInt;
				if (0 < item) & (item <= CountDItems(dlog)) then begin
					GetDKind(dlog, item, k);
					DoDialogItem := true;
				end; (* if *)
			end; (* if *)
		end; (* DoDialogItem *)

		function DoDialogItemHighlight: boolean;
			var
				res: boolean;
				dlog: DialogPtr;
				item: integer;
				k: integer;
		begin
			res := false;
			if DoDialogItem(dlog, item, k) then begin
				if (4 <= k) & (k <= 7) then begin
					res := (GetDControlHandle(dlog, item)^^.contrlHilite = GetInt);
				end;
			end; (* if *)
			DoDialogItemHighlight := res;
		end; (* DoDialogItemHighlight *)

		function DoDialogItemValue: boolean;
			var
				res: boolean;
				dlog: DialogPtr;
				item: integer;
				k: integer;
		begin
			res := false;
			if DoDialogItem(dlog, item, k) then begin
				if (4 <= k) & (k <= 7) then begin
					res := (GetDControlHandle(dlog, item)^^.contrlValue = GetInt);
				end;
			end; (* if *)
			DoDialogItemValue := res;
		end; (* DoDialogItemValue *)

		function DoDialogItemTextEmpty: boolean;
			var
				res: boolean;
				dlog: DialogPtr;
				item: integer;
				k: integer;
				s: Str255;
		begin
			res := false;
			if DoDialogItem(dlog, item, k) then begin
				if k = editText then begin
					GetDText(dlog, item, s);
					res := s <> '';
				end;
			end; (* if *)
			DoDialogItemTextEmpty := res;
		end; (* DoDialogItemTextEmpty *)

		function SetResult (b: boolean): OSErr;
			var
				tmp: integer;
				res: Ptr;
				err: OSErr;
		begin
			res := NewPtr(1);
			if res <> nil then begin
				tmp := ord(b);
				BlockMove(Ptr(longint(@tmp) + 1), res, 1);		(* I'm scared of taking @b *)
				result := res;
				result_size := 1;
				err := noErr;
			end
			else begin
				err := MemError;
			end; (* if *)
			SetResult := err;
		end; (* SetResult *)

		var
			err: OSErr;
			selector: integer;
			b: boolean;
			km: KeyMap;
	begin
		err := noErr;
		GetKeys(km);
		if km[57] then begin
			(* Debugger; *)
		end; (* if *)
		case GetInt of
			opFrontApplicationCreator: 
				b := DoFrontApplicationCreator;
			opFrontWindowName: 
				b := DoFrontWindowName;
			opDialogItemHighlight: 
				b := DoDialogItemHighlight;
			opDialogItemValue: 
				b := DoDialogItemValue;
			opDialogItemTextEmpty: 
				b := DoDialogItemTextEmpty;
			otherwise
				err := -1;
		end; (* case *)
		if err = noErr then begin
			err := SetResult(b);
		end; (* if *)
		Main := err;
	end; (* Main *)

end. (* GenericContext *)